@@ -56,32 +56,45 @@
 #ifndef EVDEV_BRAIN_H_
 #define EVDEV_BRAIN_H_
 
+#define _XF86_ANSIC_H
+#define XF86_LIBC_H
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <linux/input.h>
-#include <xf86Xinput.h>
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
+#include <xf86Xinput.h>
+
+#ifndef BITS_PER_LONG
+#define BITS_PER_LONG		(sizeof(unsigned long) * 8)
+#endif
 
-#define BITS_PER_LONG		(sizeof(long) * 8)
 #define NBITS(x)		((((x)-1)/BITS_PER_LONG)+1)
-#define OFF(x) 			((x)%BITS_PER_LONG)
 #define LONG(x)			((x)/BITS_PER_LONG)
-#define BIT(x)			(1UL<<((x)%BITS_PER_LONG))
-#define test_bit(bit, array)    ((array[LONG(bit)] >> OFF(bit)) & 1)
+#define MASK(x)			(1UL << ((x) & (BITS_PER_LONG - 1)))
+
+#ifndef test_bit
+#define test_bit(bit, array)	(!!(array[LONG(bit)] & MASK(bit)))
+#endif
+#ifndef set_bit
+#define set_bit(bit, array)	(array[LONG(bit)] |= MASK(bit))
+#endif
+#ifndef clear_bit
+#define clear_bit(bit, array)	(array[LONG(bit)] &= ~MASK(bit))
+#endif
 
 /* 2.4 compatibility */
 #ifndef EVIOCGSW
 
 #include <sys/time.h>
 #include <sys/ioctl.h>
-#include <asm/types.h>
-#include <asm/bitops.h>
 
 #define EVIOCGSW(len)		_IOC(_IOC_READ, 'E', 0x1b, len)		/* get all switch states */
 
@@ -104,6 +117,12 @@
 #include <X11/extensions/XKB.h>
 #include <X11/extensions/XKBstr.h>
 
+/* XInput 1.4+ compatability. */
+#ifndef SendCoreEvents
+#define SendCoreEvents		59
+#define DontSendCoreEvents	60
+#endif
+
 
 /*
  * Switch events
@@ -124,43 +143,51 @@
 #define EVDEV_MAXBUTTONS	96
 
 typedef struct {
-    long	ev[NBITS(EV_MAX)];
-    long	key[NBITS(KEY_MAX)];
-    long	rel[NBITS(REL_MAX)];
-    long	abs[NBITS(ABS_MAX)];
-    long	msc[NBITS(MSC_MAX)];
-    long	led[NBITS(LED_MAX)];
-    long	snd[NBITS(SND_MAX)];
-    long	ff[NBITS(FF_MAX)];
+    unsigned long	ev[NBITS(EV_MAX)];
+    unsigned long	key[NBITS(KEY_MAX)];
+    unsigned long	rel[NBITS(REL_MAX)];
+    unsigned long	abs[NBITS(ABS_MAX)];
+    unsigned long	msc[NBITS(MSC_MAX)];
+    unsigned long	led[NBITS(LED_MAX)];
+    unsigned long	snd[NBITS(SND_MAX)];
+    unsigned long	ff[NBITS(FF_MAX)];
 } evdevBitsRec, *evdevBitsPtr;
 
 typedef struct {
     int		real_buttons;
     int		buttons;
     CARD8	map[EVDEV_MAXBUTTONS];
-    int		*state[EVDEV_MAXBUTTONS];
+    void	(*callback[EVDEV_MAXBUTTONS])(InputInfoPtr pInfo, int button, int value);
 } evdevBtnRec, *evdevBtnPtr;
 
 typedef struct {
     int		axes;
-    int		n; /* Which abs_v is current, and which is previous. */
-    int		v[2][ABS_MAX];
+    int		v[ABS_MAX];
+    int		old_x, old_y;
+    int		count;
     int		min[ABS_MAX];
     int		max[ABS_MAX];
     int		map[ABS_MAX];
-    Bool	scale;
-    int		scale_x;
-    int		scale_y;
+    int		scale[2];
     int		screen; /* Screen number for this device. */
+    Bool	use_touch;
+    Bool	touch;
+    Bool	reset_x, reset_y;
 } evdevAbsRec, *evdevAbsPtr;
 
 typedef struct {
     int		axes;
     int		v[REL_MAX];
-    CARD8	btnMap[REL_MAX][2];
+    int		count;
     int		map[REL_MAX];
+    int		btnMap[REL_MAX][2];
 } evdevRelRec, *evdevRelPtr;
 
+typedef struct {
+    int		axes;
+    int		v[ABS_MAX];
+} evdevAxesRec, *evdevAxesPtr;
+
 typedef struct {
     char	*xkb_rules;
     char	*xkb_model;
@@ -179,6 +206,7 @@ typedef struct _evdevState {
     evdevAbsPtr	abs;
     evdevRelPtr	rel;
     evdevKeyPtr	key;
+    evdevAxesPtr axes;
 } evdevStateRec, *evdevStatePtr;
 
 typedef struct _evdevDevice {
@@ -228,23 +256,22 @@ Bool evdevGetBits (int fd, evdevBitsPtr bits);
 int EvdevBtnInit (DeviceIntPtr device);
 int EvdevBtnOn (DeviceIntPtr device);
 int EvdevBtnOff (DeviceIntPtr device);
-int EvdevBtnNew(InputInfoPtr pInfo);
+int EvdevBtnNew0(InputInfoPtr pInfo);
+int EvdevBtnNew1(InputInfoPtr pInfo);
 void EvdevBtnProcess (InputInfoPtr pInfo, struct input_event *ev);
 void EvdevBtnPostFakeClicks(InputInfoPtr pInfo, int button, int count);
+int EvdevBtnFind (InputInfoPtr pInfo, const char *button);
+int EvdevBtnExists (InputInfoPtr pInfo, int button);
 
-int EvdevAbsInit (DeviceIntPtr device);
-int EvdevAbsOn (DeviceIntPtr device);
-int EvdevAbsOff (DeviceIntPtr device);
-int EvdevAbsNew(InputInfoPtr pInfo);
-void EvdevAbsProcess (InputInfoPtr pInfo, struct input_event *ev);
-void EvdevAbsSyn (InputInfoPtr pInfo);
-
-int EvdevRelInit (DeviceIntPtr device);
-int EvdevRelOn (DeviceIntPtr device);
-int EvdevRelOff (DeviceIntPtr device);
-int EvdevRelNew(InputInfoPtr pInfo);
-void EvdevRelProcess (InputInfoPtr pInfo, struct input_event *ev);
-void EvdevRelSyn (InputInfoPtr pInfo);
+int EvdevAxesInit (DeviceIntPtr device);
+int EvdevAxesOn (DeviceIntPtr device);
+int EvdevAxesOff (DeviceIntPtr device);
+int EvdevAxesNew0(InputInfoPtr pInfo);
+int EvdevAxesNew1(InputInfoPtr pInfo);
+void EvdevAxesAbsProcess (InputInfoPtr pInfo, struct input_event *ev);
+void EvdevAxesRelProcess (InputInfoPtr pInfo, struct input_event *ev);
+void EvdevAxesSynRep (InputInfoPtr pInfo);
+void EvdevAxesSynCfg (InputInfoPtr pInfo);
 
 int EvdevKeyInit (DeviceIntPtr device);
 int EvdevKeyNew (InputInfoPtr pInfo);
